import { useEffect, useCallback, useState } from 'react';
import Container from './container';
var bootstrap = function (modules) {
    var container = new Container(modules);
    var dispatch = function (nameAndMethod, payload) {
        var _a = nameAndMethod.split('/'), namespace = _a[0], methodName = _a[1];
        var combineModule = container.getModule(namespace);
        var _b = combineModule[namespace], state = _b.state, reducers = _b.reducers, effects = _b.effects;
        var rootState = container.getRootState();
        if (effects[methodName]) {
            return effects[methodName]({ state: state, payload: payload, rootState: rootState, dispatch: dispatch });
        }
        else if (reducers[methodName]) {
            var newState = reducers[methodName]({
                state: state,
                rootState: rootState,
                payload: payload,
            });
            container.setState(namespace, newState);
        }
    };
    var injectFns = function (reducersOrEffects) {
        Object.keys(reducersOrEffects).forEach(function (key) {
            if (!dispatch[key])
                dispatch[key] = {};
            var originFns = reducersOrEffects[key];
            var fns = {};
            Object.keys(originFns).forEach(function (fnKey) {
                fns[fnKey] = function (payload) {
                    return dispatch(key + "/" + fnKey, payload);
                };
            });
            Object.assign(dispatch[key], fns);
        });
    };
    function useModule(namespace) {
        var _a = useState({}), setState = _a[1];
        var setStateProxy = useCallback(function () { return setState({}); }, [setState]);
        useEffect(function () {
            container.addListener(namespace, setStateProxy);
            return function () { return container.removeListener(namespace, setStateProxy); };
        }, [namespace, setStateProxy]);
        return container.getState(namespace);
    }
    var rootReducers = container.getRootReducers();
    var rootEffects = container.getRootEffects();
    injectFns(rootReducers);
    injectFns(rootEffects);
    return { useModule: useModule, dispatch: dispatch };
};
export default bootstrap;
